@@ -106,20 +106,12 @@ static char
 	debug_print (DB_DEBUG, "!! Opening %s...\n", file);
 	
 	channel = yelp_io_channel_new_file (file, &error);
-	/* TODO: Actually handle the errors sanely.  Don't crash */
 	if (!channel) {
-	  g_error ("Error opening file %s: %s\n", file, error->message);
-	  g_error_free (error);
-	  error = NULL;
-	  exit (655);
+	  return NULL;
 	}
 	result = g_io_channel_read_to_end (channel, &str, &len, &error);
-	/* TODO: Ditto above */
 	if (result != G_IO_STATUS_NORMAL) {
-	  g_error ("Error reading file: %s\n", error->message);
-	  g_error_free (error);
-	  error = NULL;
-	  exit (666);
+	  return NULL;
 	}
 	g_io_channel_shutdown (channel, FALSE, NULL);
 	g_io_channel_unref (channel);
@@ -191,7 +183,9 @@ static char
 		{
 		  filename = find_info_part (items[0], file);
 		  str = open_info_file (filename);
-		  
+		  if (!str) {
+		  	return NULL;
+		  }
 			pages = g_strsplit (str, "", 2);
 			g_free (str);
 
@@ -559,6 +553,9 @@ GtkTreeStore
 	TagTableFix *ttf;
 	
 	str = open_info_file (file);
+	if (!str) {
+		return NULL;
+	}	
 	page_list = g_strsplit (str, "\n", 0);
 
 	g_free (str);
@@ -602,6 +599,9 @@ GtkTreeStore
 		  debug_print (DB_DEBUG, "Have the indirect mapping table\n");
 			chained_info = TRUE;
 			str = process_indirect_map (*ptr, file);
+			if (!str) {
+				return NULL;
+			}
 		}
 	}
 
@@ -985,6 +985,14 @@ info_process_text_notes (xmlNodePtr *node, gchar *content, GtkTreeStore *tree)
       }
       /* If we got to here, we now gotta parse the note reference */
 
+      if (*current_real[0] == '_') {
+	/* Special type of note that isn't really a note, but pretends
+	 * it is
+	 */
+	xmlNewTextChild (holder, NULL, BAD_CAST "para1",
+			 BAD_CAST g_strconcat ("*Note", *current_real, NULL));
+	continue;
+      }
       append = strchr (*current_real, ':');
       if (!append) {
 	xmlNewTextChild (holder, NULL, BAD_CAST "para1",
@@ -1012,13 +1020,12 @@ info_process_text_notes (xmlNodePtr *node, gchar *content, GtkTreeStore *tree)
 	  if (alt_append) append = alt_append;
 	  else append = alt_append1;
 	}
-	if ((alt_append && alt_append < append) || broken)
+	if ((alt_append && alt_append < append))
 	  append = alt_append;
 	if (alt_append1 && alt_append1 < append)
 	  append = alt_append1;
       }
       append++;
-
       url = g_strndup (*current_real, append - (*current_real));
 
       /* By now, we got 2 things.  First, is append which is the (hopefully)
@@ -1088,7 +1095,6 @@ info_process_text_notes (xmlNodePtr *node, gchar *content, GtkTreeStore *tree)
 
 	    new_url = g_strdup (lurl);
 	    info = strstr (new_url, ".info)");
-	    g_free (lurl);
 	    stripped = g_strndup (new_url, info-new_url);
 	    info +=5;
 	    lurl = g_strconcat (stripped, info, NULL);
@@ -1105,7 +1111,10 @@ info_process_text_notes (xmlNodePtr *node, gchar *content, GtkTreeStore *tree)
 	gchar *frag;
 
 	tmp1 = strchr (url, ':');
-	frag = g_strndup (url, tmp1 - url);
+	if (!tmp1)
+	  frag = g_strdup (url);
+	else 
+	  frag = g_strndup (url, tmp1 - url);
 	g_strstrip (frag);
 	gtk_tree_model_foreach (GTK_TREE_MODEL (tree), resolve_frag_id, &frag);
 	href = g_strconcat ("#", frag, NULL);
@@ -1117,13 +1126,17 @@ info_process_text_notes (xmlNodePtr *node, gchar *content, GtkTreeStore *tree)
 	else {
 	  gchar *spacing = *ulink;
 	  gchar *tmp;
+	  gint count = 0;
 	  while (*spacing == ' ') {
 	    spacing++;
+	    count++;
 	  }
 	  if (spacing != *ulink) {
-	    spacing-=2;
+	    if (count > 1)
+	      spacing-=2;
 	    tmp = g_strndup (*ulink, spacing-*ulink);
-	    spacing+=2;
+	    if (count > 1)
+	      spacing+=2;
 	    xmlNewTextChild (holder, NULL, BAD_CAST "spacing",
 			     BAD_CAST tmp);
 	    g_free (tmp);
