@@ -66,11 +66,7 @@ typedef struct _SearchContainer SearchContainer;
 
 #define YELP_SEARCH_PAGER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), YELP_TYPE_SEARCH_PAGER, YelpSearchPagerPriv))
 
-/* TRANSLATORS:  If you know of a forum in your own language, please 
- * change this URL to the type required for a search of the site.
- * The %s argument is the search terms
- */
-#define ONLINE_URL N_("http://gnomesupport.org/forums/search.php?search_keywords=%s&search_cat=1")
+#define ONLINE_URL "http://api.gnome.org/yelp/forums?query=%s"
 
 #define ONLINE_NAME N_("the GNOME Support Forums")
 
@@ -88,6 +84,11 @@ struct _YelpSearchPagerPriv {
     GPtrArray *hits;
     int   snippet_request_count;
     GSList * pending_searches;
+
+    guint search_process_id;
+    guint slow_search_setup_process_id;
+    guint slow_search_process_id;
+    guint xslt_process_id;
 };
 
 enum {
@@ -261,11 +262,39 @@ search_pager_init (YelpSearchPager *pager)
     priv->search_terms          = NULL;
     priv->hits                  = NULL;
     priv->snippet_request_count = 0;
+    priv->search_process_id = 0;
+    priv->slow_search_setup_process_id = 0;
+    priv->slow_search_process_id = 0;
+    priv->xslt_process_id = 0;
+
 }
 
 static void
 search_pager_dispose (GObject *object)
-{
+{     
+    YelpSearchPager *pager = YELP_SEARCH_PAGER (object);
+    YelpSearchPagerPriv *priv = pager->priv;
+ 
+    if (priv->search_process_id != 0) {
+        g_source_remove (priv->search_process_id);
+	priv->search_process_id = 0;
+    }
+
+    if (priv->xslt_process_id != 0) {
+        g_source_remove (priv->xslt_process_id);
+	priv->xslt_process_id = 0;
+    }
+
+    if (priv->slow_search_process_id != 0) {
+        g_source_remove (priv->slow_search_process_id);
+	priv->slow_search_process_id = 0;
+    }
+ 
+    if (priv->slow_search_setup_process_id != 0) {
+       g_source_remove (priv->slow_search_setup_process_id);
+	priv->slow_search_setup_process_id = 0;
+    }
+
     G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -368,6 +397,10 @@ search_pager_cancel (YelpPager *pager)
 static void
 search_pager_finish (YelpPager   *pager)
 {
+    YelpSearchPager *spager = YELP_SEARCH_PAGER (pager);
+    YelpSearchPagerPriv *priv = spager->priv;
+
+    g_return_if_fail (priv->search_process_id == 0);
     debug_print (DB_FUNCTION, "entering\n");
     yelp_pager_set_state (pager, YELP_PAGER_STATE_FINISHED);
 }
@@ -375,6 +408,7 @@ search_pager_finish (YelpPager   *pager)
 gboolean
 search_pager_process (YelpPager *pager)
 {
+    YelpSearchPagerPriv *priv = (YELP_SEARCH_PAGER (pager))->priv;
     debug_print (DB_FUNCTION, "entering\n");
 
     yelp_pager_set_state (pager, YELP_PAGER_STATE_PARSING);
@@ -384,9 +418,10 @@ search_pager_process (YelpPager *pager)
     yelp_pager_set_state (pager, YELP_PAGER_STATE_RUNNING);
     g_signal_emit_by_name (pager, "start");
 
-    gtk_idle_add_priority (G_PRIORITY_LOW,
-			   (GtkFunction) search_pager_process_idle,
-			   pager);
+    priv->search_process_id = 
+	g_idle_add_full (G_PRIORITY_LOW,
+			 (GSourceFunc) search_pager_process_idle,
+			 pager, NULL);
     return FALSE;
 }
 
@@ -410,10 +445,14 @@ search_pager_get_sections (YelpPager *pager)
 static void
 check_finished (YelpSearchPager *pager)
 {
-    if (pager->priv->snippet_request_count == 0) {
-	gtk_idle_add_priority (G_PRIORITY_LOW,
-			       (GtkFunction) process_xslt,
-			       pager);
+    YelpSearchPagerPriv *priv = pager->priv;
+
+    if (priv->snippet_request_count == 0 &&
+        priv->xslt_process_id == 0) {
+	priv->xslt_process_id = 
+	    g_idle_add_full (G_PRIORITY_LOW,
+			     (GSourceFunc) process_xslt,
+			     pager, NULL);
     }
 }
 
@@ -674,12 +713,18 @@ search_pager_process_idle (YelpSearchPager *pager)
 #ifdef ENABLE_BEAGLE
     if (beagle_client == NULL) {
 #endif
-	gtk_idle_add ((GtkFunction) slow_search_setup,
-		      pager);
+	g_return_val_if_fail (priv->slow_search_setup_process_id == 0, FALSE);
+
+	priv->slow_search_setup_process_id = 
+	    g_idle_add ((GSourceFunc) slow_search_setup,
+		        pager);
 #ifdef ENABLE_BEAGLE
     }
 #endif
 
+    /* returning false removes this idle function from the main loop; 
+     * we also set our search process id to zero */
+    priv->search_process_id = 0; 
     return FALSE;
 }
 
@@ -819,6 +864,11 @@ process_xslt (YelpSearchPager *pager)
     }
 
     g_signal_emit_by_name (pager, "finish");
+    
+    /* returning false removes this idle function from the main loop; 
+     * we also set our xslt process id to zero to indicate it has been
+     * removed */
+    priv->xslt_process_id = 0; 
     return FALSE;
 }
 
@@ -1271,14 +1321,16 @@ build_lists (gchar *search_terms, gchar ***terms, gint **dups,
     } else {
 	suffixes = NULL;
     }
-
+    search_terms = g_strdelimit (search_terms, ":", ' ');
     list_copy = g_strsplit (g_utf8_casefold (g_strstrip (
 					  search_terms), -1),
 			    " ", -1);
     
     for (iter = list_copy; *iter != NULL; iter++) {
 	gboolean ignoring = FALSE;
-	
+	if (strlen (*iter) == 0) {
+	    continue;
+	}
 	if (g_str_has_suffix (*iter, "?")) {
 	    gchar *tmp;
 	    tmp = g_strndup (*iter, strlen (*iter) - 1);
@@ -1380,6 +1432,7 @@ build_lists (gchar *search_terms, gchar ***terms, gint **dups,
 static gboolean
 slow_search_setup (YelpSearchPager *pager)
 {
+    YelpSearchPagerPriv *priv = pager->priv;
     gchar  *content_list;
     gchar  *stderr_str;
     gchar  *lang;
@@ -1398,6 +1451,15 @@ slow_search_setup (YelpSearchPager *pager)
     else
 	lang = "C";
     
+    if (!strcmp (pager->priv->search_terms, "")) {
+	pager->priv->slow_search_setup_process_id = 0;
+	priv->xslt_process_id =
+	    g_idle_add_full (G_PRIORITY_LOW,
+			     (GSourceFunc) process_xslt,
+			     pager, NULL);
+	return FALSE;
+    }
+
     command = g_strconcat("scrollkeeper-get-content-list ", lang, NULL);
     
     if (g_spawn_command_line_sync (command, &content_list, &stderr_str, NULL, NULL)) {
@@ -1422,7 +1484,6 @@ slow_search_setup (YelpSearchPager *pager)
 				&dup_list, &stop_list, 
 				&required_no);
 
-
     while (omf_pending) {
 	GSList  *first = NULL;
 	gchar   *file  = NULL;
@@ -1560,12 +1621,17 @@ slow_search_setup (YelpSearchPager *pager)
 	    xmlFreeDoc (omf_doc);
 
     }
+    g_return_val_if_fail (priv->slow_search_process_id == 0, FALSE);
+    priv->slow_search_process_id =
+        g_idle_add ((GSourceFunc) slow_search_process, pager);
 
-    gtk_idle_add ((GtkFunction) slow_search_process,
-		  pager);
     if (parser)
 	xmlFreeParserCtxt (parser);
 
+    /* returning false removes this idle function from the main loop; 
+     * we also set our slow search _setup_ process id to zero to 
+     * indicate it has been removed */
+    priv->slow_search_setup_process_id = 0;
     return FALSE;
 
 }
@@ -1596,6 +1662,7 @@ search_free_container (SearchContainer *c)
 static gboolean 
 slow_search_process (YelpSearchPager *pager)
 {
+    YelpSearchPagerPriv *priv = pager->priv;
     SearchContainer *c;
     GSList *first = pager->priv->pending_searches;
     gint i, j=0;
@@ -1604,9 +1671,12 @@ slow_search_process (YelpSearchPager *pager)
 	g_slist_remove_link (pager->priv->pending_searches, first);
 
     if (first == NULL) {
-	gtk_idle_add_priority (G_PRIORITY_LOW,
-			       (GtkFunction) process_xslt,
-			       pager);
+	g_return_val_if_fail (priv->xslt_process_id == 0, FALSE);
+	priv->xslt_process_id = 
+	    gtk_idle_add_priority (G_PRIORITY_LOW,
+				   (GtkFunction) process_xslt,
+				   pager);
+	priv->slow_search_process_id = 0;
 	return FALSE;
     }
 
@@ -1649,9 +1719,17 @@ slow_search_process (YelpSearchPager *pager)
 #endif
 	search_free_container (c);
 
-	gtk_idle_add_priority (G_PRIORITY_LOW,
-			       (GtkFunction) process_xslt,
-			       pager);
+	g_return_val_if_fail (priv->xslt_process_id == 0, FALSE);
+
+	priv->xslt_process_id =
+	    g_idle_add_full (G_PRIORITY_LOW,
+			     (GSourceFunc) process_xslt,
+			     pager, NULL);
+	
+	/* returning false removes this idle function from the main loop; 
+     	 * we also set our slow search process id to zero to 
+     	 * indicate it has been removed */
+    	priv->slow_search_process_id = 0;
 	return FALSE;
     }
 }
@@ -1801,7 +1879,7 @@ process_man_result (YelpSearchPager *pager, gchar *result, gchar **terms)
 	gchar *title = NULL;
 	/*gint i;*/
 
-	if (line == NULL || line[0] == NULL)
+	if (line == NULL || line[0] == NULL || line[1] == NULL)
 	    continue;
 
 	title = g_strdup (g_strstrip (line[0]));
