@@ -33,15 +33,10 @@
 #include <libgnome/gnome-init.h>
 
 #include "yelp-utils.h"
+#include "yelp-debug.h"
 
 #include <string.h>
 
-#ifdef YELP_DEBUG
-#define d(x) x
-#else
-#define d(x)
-#endif
-
 static GHashTable *doc_info_table;
 
 typedef struct {
@@ -132,8 +127,8 @@ yelp_doc_info_new (const gchar *uri, gboolean trust_uri)
 
     g_return_val_if_fail (uri != NULL, NULL);
 
-    d (g_print ("yelp_doc_info_new\n"));
-    d (g_print ("  uri      = \"%s\"\n", uri));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  uri      = \"%s\"\n", uri);
 
     if (trust_uri)
 	full_uri = gnome_vfs_make_uri_from_input (uri);
@@ -164,8 +159,18 @@ yelp_doc_info_new (const gchar *uri, gboolean trust_uri)
     }
     else if (g_str_has_prefix (full_uri, "info:")) {
 	doc_uri  = convert_info_uri (full_uri);
-	doc_type = YELP_DOC_TYPE_INFO;
-	uri_type = YELP_URI_TYPE_INFO;
+	if (!g_str_has_prefix (doc_uri, "man:")) {
+	    doc_type = YELP_DOC_TYPE_INFO;
+	    uri_type = YELP_URI_TYPE_INFO;
+	} else {
+	    gchar *tmp;
+	    tmp = g_strdup (doc_uri);
+	    g_free (doc_uri);
+	    doc_uri = convert_man_uri (tmp, trust_uri);
+	    g_free (tmp);
+	    doc_type = YELP_DOC_TYPE_MAN;
+	    uri_type = YELP_URI_TYPE_MAN;	    
+	}
     }
     else if (g_str_has_prefix (full_uri, "x-yelp-toc:")) {
 	doc_uri = g_strdup ("file://" DATADIR "/yelp/toc.xml");
@@ -182,9 +187,9 @@ yelp_doc_info_new (const gchar *uri, gboolean trust_uri)
 	doc_type = YELP_DOC_TYPE_EXTERNAL;
 	uri_type = YELP_URI_TYPE_EXTERNAL;
     }
-
-    d (g_print ("  full_uri = \"%s\"\n", full_uri));
-    d (g_print ("  doc_uri  = \"%s\"\n", doc_uri));
+    
+    debug_print (DB_ARG, "  full_uri = \"%s\"\n", full_uri);
+    debug_print (DB_ARG, "  doc_uri  = \"%s\"\n", doc_uri);
 
     g_free (full_uri);
 
@@ -216,8 +221,8 @@ yelp_doc_info_get (const gchar *uri, gboolean trust_uri)
 
     g_return_val_if_fail (uri != NULL, NULL);
 
-    d (g_print ("yelp_doc_info_get\n"));
-    d (g_print ("  uri     = \"%s\"\n", uri));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  uri     = \"%s\"\n", uri);
 
     if (!doc_info_table)
 	doc_info_table =
@@ -296,8 +301,8 @@ yelp_doc_info_free (YelpDocInfo *doc)
 {
     gint i;
 
-    d (g_print ("yelp_doc_info_free\n"));
-    d (g_print ("  uri = \"%s\"\n", doc->uris->uri));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  uri = \"%s\"\n", doc->uris->uri);
 
     if (!doc)
 	return;
@@ -596,7 +601,7 @@ yelp_doc_info_add_uri (YelpDocInfo *doc_info,
 {
     DocInfoURI *info_uri;
 
-    d (g_print ("yelp_doc_add_uri\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     g_assert (doc_info->num_uris <= doc_info->max_uris);
 
@@ -614,9 +619,9 @@ yelp_doc_info_add_uri (YelpDocInfo *doc_info,
 
     doc_info->num_uris++;
 
-    d (g_print ("  uri      = \"%s\"\n", uri));
-    d (g_print ("  num_uris = %i\n", doc_info->num_uris));
-    d (g_print ("  max_uris = %i\n", doc_info->max_uris));
+    debug_print (DB_ARG, "  uri      = \"%s\"\n", uri);
+    debug_print (DB_ARG, "  num_uris = %i\n", doc_info->num_uris);
+    debug_print (DB_ARG, "  max_uris = %i\n", doc_info->max_uris);
 }
 
 /******************************************************************************/
@@ -783,7 +788,9 @@ convert_man_uri (gchar *uri, gboolean trust_uri)
 
 	if (!g_spawn_command_line_sync ("manpath", &manp, NULL, NULL, NULL))
 	    manp = g_strdup (g_getenv ("MANPATH"));
-
+	if (!manp) {
+	    return NULL;
+	}
 	g_strstrip (manp);
 	manpath = g_strsplit (manp, ":", -1);
 
@@ -897,6 +904,8 @@ convert_info_uri (gchar   *uri)
     gchar *info_name = NULL;
     gchar *info_dot_info = NULL;
     gchar **infopaths = NULL;
+    gboolean need_subdir = FALSE;
+    gchar *test_filename = NULL;
 
     gint i;
 
@@ -905,6 +914,7 @@ convert_info_uri (gchar   *uri)
     gchar *pattern;
     GPatternSpec *pspec = NULL;
     GPatternSpec *pspec1 = NULL;
+    gchar *subdir = NULL;
 
     if ((path = strchr(uri, ':')))
 	path++;
@@ -941,6 +951,20 @@ convert_info_uri (gchar   *uri)
     else
 	info_name = g_strdup (path);
 
+    if (strstr (info_name, "/")) {
+	gchar *tmp = NULL;
+	gchar *real_name = NULL;
+	tmp = strstr (info_name, "/");
+	tmp++;
+	real_name = g_strdup (tmp);
+	subdir = g_strndup (info_name, (strstr (info_name, "/") - info_name));
+	g_free (info_name);
+	info_name = g_strdup (real_name);
+	g_free (real_name);
+	need_subdir = TRUE;
+    }
+
+
     pattern = g_strdup_printf ("%s.info.*", info_name);
     pspec = g_pattern_spec_new (pattern);
     g_free (pattern);
@@ -956,14 +980,59 @@ convert_info_uri (gchar   *uri)
 	dir = g_dir_open (infopath[i], 0, NULL);
 	if (dir) {
 	    while ((filename = g_dir_read_name (dir))) {
-		if (g_str_equal (info_dot_info, filename)  ||
-		    g_pattern_match_string (pspec, filename) ||
-		    g_pattern_match_string (pspec1, filename) ||
-		    g_str_equal (info_name, filename)) {
-		    doc_uri = g_strconcat ("file://",
-					   infopath[i], "/",
-					   filename,
-					   NULL);
+		g_free (test_filename);
+		test_filename =  g_strconcat (infopath[i], "/", filename, NULL);
+		if (need_subdir && g_str_equal (filename, subdir)) {
+		    gchar *dirname = NULL;
+		    g_dir_close (dir);
+		    dirname = g_strconcat (infopath[i], "/", subdir, NULL);
+		    dir = g_dir_open (dirname, 0, NULL);
+		    g_free (dirname);
+		    filename = g_dir_read_name (dir);
+		    need_subdir = FALSE;
+		}
+		else if (g_str_equal (filename,info_name) && 
+			 g_file_test (test_filename, G_FILE_TEST_IS_DIR)) {
+		    /* In dir, they've specified the subdir but not the
+		     * info file name.  Here, do some work to get the name
+		     *  ...*/
+		    gchar *real_filename;
+
+		    g_dir_close (dir);
+		    dir = g_dir_open (test_filename, 0, NULL);
+
+		     while ((real_filename = g_dir_read_name (dir))) {
+			 if ((g_str_equal (info_dot_info, real_filename)  ||
+			  g_pattern_match_string (pspec, real_filename) ||
+			  g_pattern_match_string (pspec1, real_filename) ||
+			      g_str_equal (info_name, real_filename))) {
+			     doc_uri = g_strconcat ("file://",
+						    test_filename, "/",
+						    real_filename,
+						    NULL);
+			     g_dir_close (dir);
+			     goto done;
+			 }
+		     }
+
+		}
+		else if (!need_subdir && 
+			 (g_str_equal (info_dot_info, filename)  ||
+			  g_pattern_match_string (pspec, filename) ||
+			  g_pattern_match_string (pspec1, filename) ||
+			  g_str_equal (info_name, filename))) {
+		    if (subdir) {
+			doc_uri = g_strconcat ("file://",
+					       infopath[i], "/", subdir, "/",
+					       filename,
+					       NULL);
+		    } else {
+			doc_uri = g_strconcat ("file://",
+					       infopath[i], "/",
+					       filename,
+					       NULL);
+
+		    }
 		    g_dir_close (dir);
 		    goto done;
 		}
@@ -972,12 +1041,24 @@ convert_info_uri (gchar   *uri)
 	}
     }
 
+    /* If we got this far and doc_uri is still NULL, we resort to looking
+     * for a man page.  Let above handle that.  We just let it know that 
+     * somethings gotta be done */
+    if (doc_uri == NULL) {
+	gchar *tmp;
+	tmp = strchr (uri, ':');
+	doc_uri = g_strconcat ("man",tmp,NULL);
+    }
+
  done:
     if (pspec)
 	g_pattern_spec_free (pspec);
     if (pspec1)
 	g_pattern_spec_free (pspec1);
+    g_free (test_filename);
+    g_free (subdir);
     g_free (info_dot_info);
     g_free (info_name);
+    
     return doc_uri;
 }
